/*
 * Copyright (c) 1997, 2018 Oracle and/or its affiliates. All rights reserved.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Distribution License v. 1.0, which is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

package com.sun.xml.ws.rx.util;

/**
 * This interface can implemented by any class that is able to provide time-related 
 * information. Using implementation of this class may especially be usefull in cases
 * when we need to provide synchronized time information in a clustered or otherwise 
 * distributed environment.
 *
 * @author Marek Potociar <marek.potociar at sun.com>
 */
public interface TimeSynchronizer {
    /*
     * Returns the current time in milliseconds.  Note that
     * while the unit of time of the return value is a millisecond,
     * the granularity of the value depends on the underlying
     * synchronization system and may be larger.  For example, many
     * operating systems measure time in units of tens of
     * milliseconds.
     * <p />
     *
     * @return  the difference, measured in milliseconds, between
     *          the current time and midnight, January 1, 1970 UTC.
     */
    public long currentTimeInMillis();
}
